
-- helper function:
-- create a formspec dropdown list with player names (first entry: Add player) and
-- an option to delete players from that list
-- Note: With the what_is_the_list_about-parameter, it is possible to handle i.e. variables as well
yl_speak_up.create_dropdown_playerlist = function(player, pname,
		table_of_names, index_selected,
		start_x, start_y, stretch_x, h, dropdown_name, what_is_the_list_about, delete_button_text,
		field_name_for_adding_player, explain_add_player,
		field_name_for_deleting_player, explain_delete_player)

	local text = "dropdown["..tostring(start_x)..","..tostring(start_y)..";"..
				tostring(3.8 + stretch_x)..","..tostring(h)..";"..
				tostring(dropdown_name)..";Add "..tostring(what_is_the_list_about)..":"
	-- table_of_names is a table with the playernames as keys
	-- we want to work with indices later on; in order to be able to do that reliably, we
	-- need a defined order of names
	local tmp_list = yl_speak_up.sort_keys(table_of_names, true)
	for i, p in ipairs(tmp_list) do
		text = text..","..minetest.formspec_escape(p)
	end
	-- has an entry been selected?
	if(not(index_selected) or index_selected < 0 or index_selected > #tmp_list+1) then
		index_selected = 1
	end
	text = text..";"..tostring(index_selected)..";]"
	if(index_selected == 1) then
		-- first index "Add player" selected? Then offer a field for entering the name
		text = text.."field["..tostring(start_x + 4.0 + stretch_x)..","..tostring(start_y)..
				";"..tostring(3.5 + stretch_x)..","..tostring(h)..";"..
				tostring(field_name_for_adding_player)..";;]"..
				"tooltip["..tostring(field_name_for_adding_player)..";"..
					tostring(explain_add_player).."]"
	else
		text = text.."button["..tostring(start_x + 3.8 + stretch_x)..","..tostring(start_y)..
				";"..tostring(3.4 + stretch_x)..","..tostring(h)..";"..
				tostring(field_name_for_deleting_player)..";"..
				tostring(delete_button_text).."]"..
				"tooltip["..tostring(field_name_for_deleting_player)..";"..
					tostring(explain_delete_player).."]"
	end
	return text
end



-- inserts buttons into formspec which allow to select previous/next entry, to go back,
-- create new entries, delete entries and select entries from a dropdown menu;
-- returns the currently selected entry or nil (=create new entry)
-- Note: Designed for a formspec of size "size[18,12]"
yl_speak_up.get_fs_manage_general = function(player, param,
			formspec, list_of_entries,
			text_add_new, tooltip_add_new,
			what_is_the_list_about,
			tooltip_add_entry_general, tooltip_del_entry_general)
	local selected = nil
	local pname = player:get_player_name()
	-- the yl_speak_up.create_dropdown_playerlist function needs a table - not a list
	local table_of_entries = {}
	for i, k in ipairs(list_of_entries) do
		table_of_entries[ k ] = true
	end
	-- "Add variable:" is currently selected
	if(not(yl_speak_up.speak_to[pname].tmp_index_general)
	   or yl_speak_up.speak_to[pname].tmp_index_general == 1
	   or not(list_of_entries[ yl_speak_up.speak_to[pname].tmp_index_general - 1])) then
		yl_speak_up.speak_to[pname].tmp_index_general = 1
		table.insert(formspec,	"button[12.2,2.15;2.5,0.6;add_list_entry;")
		table.insert(formspec,	minetest.formspec_escape(text_add_new))
		table.insert(formspec,	"]")
		table.insert(formspec,	"tooltip[add_list_entry;")
		table.insert(formspec,	minetest.formspec_escape(tooltip_add_new))
		table.insert(formspec,	"]")
	else
		-- index 1 is "Add variable:"
		selected = list_of_entries[ yl_speak_up.speak_to[pname].tmp_index_general - 1]
	end
	if(yl_speak_up.speak_to[pname].tmp_index_general > 1) then
		table.insert(formspec,	"button[4.0,0.2;2.0,0.6;prev;< Prev]"..
					"button[4.0,11.0;2.0,0.6;prev;< Prev]")
	end
	if(yl_speak_up.speak_to[pname].tmp_index_general <= #list_of_entries) then
		table.insert(formspec,	"button[12.0,0.2;2.0,0.6;next;Next >]"..
					"button[12.0,11.0;2.0,0.6;next;Next >]")
	end
	table.insert(formspec, 		"button[0.0,0.2;2.0,0.6;back;Back]"..
					"button[8.0,11.0;2.0,0.6;back;Back]")
	local what = minetest.formspec_escape(what_is_the_list_about)
	table.insert(formspec,		"label[7.0,0.4;* Manage your ")
	table.insert(formspec,		what)
	table.insert(formspec,		"s *]")
	table.insert(formspec,		"label[0.2,2.45;Your ")
	table.insert(formspec,		what)
	table.insert(formspec,		":]")
	-- offer a dropdown list and a text input field for new varialbe names for adding
	table.insert(formspec,		yl_speak_up.create_dropdown_playerlist(
						player, pname,
						table_of_entries,
						yl_speak_up.speak_to[pname].tmp_index_general,
						2.6, 2.15, 1.0, 0.6,
						"list_of_entries",
						what,
						"Delete selected "..what,
						"add_entry_general",
						minetest.formspec_escape(tooltip_add_entry_general),
						"del_entry_general",
						minetest.formspec_escape(tooltip_del_entry_general)
					))

	-- either nil or the text of the selected entry
	return selected
end
